@@ -21,6 +21,7 @@ from .attachment import Attachment
 from .attachment import client as attachment_client
 from .attachment import converters as attachment_converters
 from .dataset import rest_operations as dataset_rest_operations
+from .experiment import experiments_client
 from .experiment import helpers as experiment_helpers
 from .experiment import rest_operations as experiment_rest_operations
 from .prompt import Prompt, PromptType
@@ -37,7 +38,12 @@ from .. import (
     rest_client_configurator,
     url_helpers,
 )
-from ..message_processing import messages, streamer_constructors, message_queue
+from ..message_processing import (
+    messages,
+    streamer_constructors,
+    message_queue,
+    message_processors_chain,
+)
 from ..message_processing.batching import sequence_splitter
 from ..rest_api import client as rest_api_client
 from ..rest_api.core.api_error import ApiError
@@ -175,6 +181,11 @@ class Opik:
             batch_factor=self._config.maximal_queue_size_batch_factor,
         )
 
+        self._message_processor = (
+            message_processors_chain.create_message_processors_chain(
+                rest_client=self._rest_client
+            )
+        )
         self._streamer = streamer_constructors.construct_online_streamer(
             n_consumers=workers,
             rest_client=self._rest_client,
@@ -182,6 +193,7 @@ class Opik:
             use_batching=use_batching,
             file_upload_worker_count=file_upload_worker_count,
             max_queue_size=max_queue_size,
+            message_processor=self._message_processor,
         )
 
     def _display_trace_url(self, trace_id: str, project_name: str) -> None:
@@ -703,6 +715,38 @@ class Opik:
 
             self._streamer.put(add_trace_feedback_scores_batch_message)
 
+    def log_threads_feedback_scores(
+        self, scores: List[FeedbackScoreDict], project_name: Optional[str] = None
+    ) -> None:
+        """
+        Log feedback scores for threads.
+
+        Args:
+            scores (List[FeedbackScoreDict]): A list of feedback score dictionaries.
+                Specifying a thread id via `id` key for each score is mandatory.
+            project_name: The name of the project in which the threads are logged. If not set, the project name
+                which was configured when the Opik instance was created will be used.
+
+        Returns:
+            None
+
+        Example:
+            >>> from opik import Opik
+            >>> client = Opik()
+            >>> scores = [
+            >>>     {
+            >>>         "id": "thread_123",
+            >>>         "name": "user_satisfaction",
+            >>>         "value": 0.85,
+            >>>         "reason": "User seemed satisfied with the conversation"
+            >>>     }
+            >>> ]
+            >>> client.log_threads_feedback_scores(scores=scores)
+        """
+        self.get_threads_client().log_threads_feedback_scores(
+            scores=scores, project_name=project_name
+        )
+
     def delete_trace_feedback_score(self, trace_id: str, name: str) -> None:
         """
         Deletes a feedback score associated with a specific trace.
@@ -804,7 +848,7 @@ class Opik:
         )
 
         experiments = dataset_rest_operations.get_dataset_experiments(
-            self._rest_client, dataset_id, max_results
+            self._rest_client, dataset_id, max_results, streamer=self._streamer
         )
 
         return experiments
@@ -916,6 +960,7 @@ class Opik:
             name=name,
             dataset_name=dataset_name,
             rest_client=self._rest_client,
+            streamer=self._streamer,
             prompts=checked_prompts,
         )
 
@@ -943,7 +988,7 @@ class Opik:
             name=name,
             dataset_name=experiment_public.dataset_name,
             rest_client=self._rest_client,
-            # TODO: add prompt if exists
+            streamer=self._streamer,
         )
 
     def get_experiments_by_name(self, name: str) -> List[experiment.Experiment]:
@@ -964,9 +1009,10 @@ class Opik:
         for public_experiment in experiments_public:
             experiment_ = experiment.Experiment(
                 id=public_experiment.id,
-                dataset_name=public_experiment.dataset_name,
                 name=name,
+                dataset_name=public_experiment.dataset_name,
                 rest_client=self._rest_client,
+                streamer=self._streamer,
             )
             result.append(experiment_)
 
@@ -998,7 +1044,7 @@ class Opik:
             name=experiment_public.name,
             dataset_name=experiment_public.dataset_name,
             rest_client=self._rest_client,
-            # TODO: add prompt if exists
+            streamer=self._streamer,
         )
 
     def end(self, timeout: Optional[int] = None) -> None:
@@ -1455,6 +1501,15 @@ class Opik:
         _ = self._rest_client.optimizations.get_optimization_by_id(id)
         return optimization.Optimization(id=id, rest_client=self._rest_client)
 
+    def get_experiments_client(self) -> experiments_client.ExperimentsClient:
+        """
+        Retrieves an instance of `ExperimentsClient`.
+
+        Returns:
+            An instance of the ExperimentsClient initialized with a cached REST client.
+        """
+        return experiments_client.ExperimentsClient(self._rest_client)
+
 
 @functools.lru_cache()
 def get_client_cached() -> Opik:
