@@ -1,12 +1,11 @@
 import logging
 import time
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable, Dict, List, Optional, Union, cast
 
 from .. import Prompt
 from ..api_objects import opik_client
 from ..api_objects import dataset, experiment
 from ..api_objects.experiment import helpers as experiment_helpers
-from ..api_objects.prompt import prompt_template
 from . import (
     asyncio_support,
     engine,
@@ -16,17 +15,37 @@ from . import (
     samplers,
 )
 from .metrics import base_metric
-from .models import base_model, models_factory
+from .models import ModelCapabilities, base_model, models_factory
+from .scorers import scorer_function, scorer_wrapper_metric
 from .types import LLMTask, ScoringKeyMappingType
 from .. import url_helpers
+from opik.api_objects.prompt.chat_prompt_template import ChatPromptTemplate
+from opik.api_objects.prompt.types import SupportedModalities
 
 LOGGER = logging.getLogger(__name__)
+MODALITY_SUPPORT_DOC_URL = (
+    "https://www.comet.com/docs/opik/evaluation/evaluate_multimodal"
+)
+
+
+def _try_notifying_about_experiment_completion(
+    experiment: experiment.Experiment,
+) -> None:
+    try:
+        experiment.experiments_rest_client.finish_experiments(ids=[experiment.id])
+    except Exception:
+        LOGGER.debug(
+            "Failed to notify backend about the experiment completion. Experiment ID: %s",
+            experiment.id,
+            exc_info=True,
+        )
 
 
 def evaluate(
     dataset: dataset.Dataset,
     task: LLMTask,
     scoring_metrics: Optional[List[base_metric.BaseMetric]] = None,
+    scoring_functions: Optional[List[scorer_function.ScorerFunction]] = None,
     experiment_name: Optional[str] = None,
     project_name: Optional[str] = None,
     experiment_config: Optional[Dict[str, Any]] = None,
@@ -41,7 +60,9 @@ def evaluate(
     trial_count: int = 1,
 ) -> evaluation_result.EvaluationResult:
     """
-    Performs task evaluation on a given dataset.
+    Performs task evaluation on a given dataset. You can use either `scoring_metrics` or `scorer_functions` to calculate
+    evaluation metrics. The scorer functions doesn't require `scoring_key_mapping` and use reserved parameters
+    to receive inputs and outputs from the task.
 
     Args:
         dataset: An Opik dataset instance
@@ -63,6 +84,13 @@ def evaluate(
             are mandatory in `task`-returned dictionary.
             If no value provided, the experiment won't have any scoring metrics.
 
+        scoring_functions: List of scorer functions to be executed during evaluation.
+            Each scorer function includes a scoring method that accepts predefined
+            arguments supplied by the evaluation engine:
+                • dataset_item — a dictionary containing the dataset item content,
+                • task_outputs — a dictionary containing the LLM task output.
+                • task_span - the data collected during the LLM task execution [optional].
+
         verbose: an integer value that controls evaluation output logs such as summary and tqdm progress bar.
             0 - no outputs, 1 - outputs are enabled (default), 2 - outputs are enabled and detailed statistics
             are displayed.
@@ -90,9 +118,6 @@ def evaluate(
 
         trial_count: number of times to run the task and evaluate the task output for every dataset item.
     """
-    if scoring_metrics is None:
-        scoring_metrics = []
-
     checked_prompts = experiment_helpers.handle_prompt_args(
         prompt=prompt,
         prompts=prompts,
@@ -107,6 +132,13 @@ def evaluate(
         prompts=checked_prompts,
     )
 
+    # wrap scoring functions if any
+    scoring_metrics = _wrap_scoring_functions(
+        scoring_functions=scoring_functions,
+        scoring_metrics=scoring_metrics,
+        project_name=project_name,
+    )
+
     return _evaluate_task(
         client=client,
         experiment=experiment,
@@ -176,6 +208,8 @@ def _evaluate_task(
 
     client.flush()
 
+    _try_notifying_about_experiment_completion(experiment)
+
     evaluation_result_ = evaluation_result.EvaluationResult(
         dataset_id=dataset.id,
         experiment_id=experiment.id,
@@ -197,12 +231,15 @@ def _evaluate_task(
 def evaluate_experiment(
     experiment_name: str,
     scoring_metrics: List[base_metric.BaseMetric],
+    scoring_functions: Optional[List[scorer_function.ScorerFunction]] = None,
     scoring_threads: int = 16,
     verbose: int = 1,
     scoring_key_mapping: Optional[ScoringKeyMappingType] = None,
     experiment_id: Optional[str] = None,
 ) -> evaluation_result.EvaluationResult:
-    """Update the existing experiment with new evaluation metrics.
+    """Update the existing experiment with new evaluation metrics. You can use either `scoring_metrics` or `scorer_functions` to calculate
+    evaluation metrics. The scorer functions doesn't require `scoring_key_mapping` and use reserved parameters
+    to receive inputs and outputs from the task.
 
     Args:
         experiment_name: The name of the experiment to update.
@@ -213,6 +250,13 @@ def evaluate_experiment(
             of the `score` method in metrics that you need to find out which keys
             are mandatory in `task`-returned dictionary.
 
+        scoring_functions: List of scorer functions to be executed during evaluation.
+            Each scorer function includes a scoring method that accepts predefined
+            arguments supplied by the evaluation engine:
+                • dataset_item — a dictionary containing the dataset item content,
+                • task_outputs — a dictionary containing the LLM task output.
+                • task_span - the data collected during the LLM task execution [optional].
+
         scoring_threads: amount of thread workers to run scoring metrics.
 
         verbose: an integer value that controls evaluation output logs such as summary and tqdm progress bar.
@@ -247,6 +291,13 @@ def evaluate_experiment(
         client=client, trace_id=first_trace_id
     )
 
+    # wrap scoring functions if any
+    scoring_metrics = _wrap_scoring_functions(
+        scoring_functions=scoring_functions,
+        scoring_metrics=scoring_metrics,
+        project_name=project_name,
+    )
+
     with asyncio_support.async_http_connections_expire_immediately():
         evaluation_engine = engine.EvaluationEngine(
             client=client,
@@ -276,6 +327,8 @@ def evaluate_experiment(
 
     report.display_experiment_link(experiment_url=experiment_url)
 
+    _try_notifying_about_experiment_completion(experiment)
+
     evaluation_result_ = evaluation_result.EvaluationResult(
         dataset_id=experiment.dataset_id,
         experiment_id=experiment.id,
@@ -297,19 +350,40 @@ def evaluate_experiment(
 def _build_prompt_evaluation_task(
     model: base_model.OpikBaseModel, messages: List[Dict[str, Any]]
 ) -> Callable[[Dict[str, Any]], Dict[str, Any]]:
-    def _prompt_evaluation_task(prompt_variables: Dict[str, Any]) -> Dict[str, Any]:
-        processed_messages = []
-        for message in messages:
-            processed_messages.append(
-                {
-                    "role": message["role"],
-                    "content": prompt_template.PromptTemplate(
-                        message["content"],
-                        validate_placeholders=False,
-                        type=prompt_variables.get("type", "mustache"),
-                    ).format(**prompt_variables),
-                }
+    supported_modalities = cast(
+        SupportedModalities,
+        {
+            "vision": ModelCapabilities.supports_vision(
+                getattr(model, "model_name", None)
             )
+        },
+    )
+    chat_prompt_template = ChatPromptTemplate(messages=messages)
+
+    required_modalities = chat_prompt_template.required_modalities()
+    unsupported_modalities = {
+        modality
+        for modality in required_modalities
+        if not supported_modalities.get(modality, False)
+    }
+
+    if unsupported_modalities:
+        modalities_list = ", ".join(sorted(unsupported_modalities))
+        LOGGER.warning(
+            "Model '%s' does not support %s content. Multimedia parts will be flattened "
+            "to text placeholders. See %s for supported models and customization options.",
+            getattr(model, "model_name", "unknown"),
+            modalities_list,
+            MODALITY_SUPPORT_DOC_URL,
+        )
+
+    def _prompt_evaluation_task(prompt_variables: Dict[str, Any]) -> Dict[str, Any]:
+        template_type_override = prompt_variables.get("type")
+        processed_messages = chat_prompt_template.format(
+            variables=prompt_variables,
+            supported_modalities=supported_modalities,
+            template_type=template_type_override,
+        )
 
         with base_model.get_provider_response(
             model_provider=model, messages=processed_messages
@@ -327,6 +401,7 @@ def evaluate_prompt(
     messages: List[Dict[str, Any]],
     model: Optional[Union[str, base_model.OpikBaseModel]] = None,
     scoring_metrics: Optional[List[base_metric.BaseMetric]] = None,
+    scoring_functions: Optional[List[scorer_function.ScorerFunction]] = None,
     experiment_name: Optional[str] = None,
     project_name: Optional[str] = None,
     experiment_config: Optional[Dict[str, Any]] = None,
@@ -351,6 +426,13 @@ def evaluate_prompt(
         scoring_metrics: List of metrics to calculate during evaluation.
             The LLM input and output will be passed as arguments to each metric `score(...)` method.
 
+        scoring_functions: List of scorer functions to be executed during evaluation.
+            Each scorer function includes a scoring method that accepts predefined
+            arguments supplied by the evaluation engine:
+                • dataset_item — a dictionary containing the dataset item content,
+                • task_outputs — a dictionary containing the LLM task output.
+                • task_span - the data collected during the LLM task execution [optional].
+
         experiment_name: name of the experiment.
 
         project_name: The name of the project to log data
@@ -391,9 +473,6 @@ def evaluate_prompt(
         if "model" not in experiment_config:
             experiment_config["model"] = opik_model.model_name
 
-    if scoring_metrics is None:
-        scoring_metrics = []
-
     client = opik_client.get_client_cached()
 
     prompts = [prompt] if prompt else None
@@ -405,6 +484,13 @@ def evaluate_prompt(
         prompts=prompts,
     )
 
+    # wrap scoring functions if any
+    scoring_metrics = _wrap_scoring_functions(
+        scoring_functions=scoring_functions,
+        scoring_metrics=scoring_metrics,
+        project_name=project_name,
+    )
+
     start_time = time.time()
 
     with asyncio_support.async_http_connections_expire_immediately():
@@ -441,6 +527,8 @@ def evaluate_prompt(
 
     client.flush()
 
+    _try_notifying_about_experiment_completion(experiment)
+
     evaluation_result_ = evaluation_result.EvaluationResult(
         experiment_id=experiment.id,
         dataset_id=dataset.id,
@@ -562,3 +650,20 @@ def evaluate_optimization_trial(
         dataset_sampler=dataset_sampler,
         trial_count=trial_count,
     )
+
+
+def _wrap_scoring_functions(
+    scoring_metrics: Optional[List[base_metric.BaseMetric]] = None,
+    scoring_functions: Optional[List[scorer_function.ScorerFunction]] = None,
+    project_name: Optional[str] = None,
+) -> List[base_metric.BaseMetric]:
+    if scoring_functions:
+        function_metrics = scorer_wrapper_metric.wrap_scorer_functions(
+            scoring_functions, project_name=project_name
+        )
+        if scoring_metrics:
+            scoring_metrics.extend(function_metrics)
+        else:
+            scoring_metrics = function_metrics
+
+    return scoring_metrics if scoring_metrics else []
